unit Generic;

interface

uses syncobjs,sysutils;

const
   cryptoBufferSize = 1024;
   bufsize = 1024 * 1024 * 5;
   audioBufferSize = 4096; // for 8-bit use 4096, for 16-bit use 8192
   logFileName = 'cryptospeak.log';

var
   createLogFile: boolean;
   threadLockLogFile: TCriticalSection;
   logfile: text;

function calcChecksum(buffer: pbyte; count: integer): byte;
procedure logToFile(msg: string);
procedure logBytes(logmsg: string; buffer: pbyte; count: integer);

implementation

function calcChecksum(buffer: pbyte; count: integer): byte;
var
   i: integer;
begin
   result := 0;
   for i := 1 to Count do
   begin
      result := result + buffer^;
      result := result and 255;
      inc(buffer);
   end;
end;

procedure logToFile(msg: string);
begin
   if createLogFile then
   begin
      threadLockLogFile.enter;
      try
         assignfile(logfile, logfilename);
         append(logfile);
         writeln(logfile, msg);
         closefile(logfile);
      finally
         threadLockLogFile.leave;
      end;
   end;
end;



procedure logBytes(logmsg: string; buffer: pbyte; count: integer);
var
   bytesAsString: string;
   i: integer;
begin
   if createLogFile then
   begin
      bytesAsString := '';
      for i := 0 to count - 1 do
      begin
         if (i < 5) or (i > (count-5)) then
            bytesAsString := bytesAsString + inttostr(buffer^) + '/'
         else if (i = 10) then
            bytesAsString := bytesAsString + '....';
         inc(buffer);
      end;
      logToFile(logmsg + bytesAsString + ' (' + inttostr(count) + ')');
   end;
end;


initialization
threadLockLogFile := TCriticalSection.create;

finalization
threadLockLogFile.free;

end.
