unit Senderthread;

interface

uses
  Classes, Windows, SysUtils, forms, stdctrls,ScktComp2,
  DCPrijndael,zlib,cyclicBuffer,generic;

type
  TSenderThread = class(TThread)
  private
     sendCipher: TDCP_rijndael;
     outputStream : TWinSocketStream;
     rawSendBuffer: TCyclicBuffer;
     frmOwner: TForm;
  protected
    procedure Execute; override;
  public
    bytesRequired: integer;
    constructor create(_frmOwner: TForm; _outputStream: TWinSocketStream; _sendCipher: TDCP_rijndael);
    destructor Destroy; override;
    function processBytesFromMicrophone(buffer: pchar; var size: integer): boolean;
    procedure clearBuffer;
  end;

implementation

uses unit1;


{constructor}
constructor TSenderThread.create(_frmOwner: TForm; _outputStream: TWinSocketStream; _sendCipher: TDCP_rijndael);
begin
   inherited create(true); // create but don't start running yet
   frmOwner := _frmOwner;
   sendCipher := _sendCipher;
   outputStream := _outputStream;
   rawSendBuffer := TCyclicBuffer.create(bufsize);
end;

{destructor}
destructor TSenderThread.Destroy;
begin
   rawSendBuffer.free;
   inherited destroy;
end;


{main thread entry point}
procedure TSenderThread.Execute;
var
   sendCryptobufferIn, sendCryptobufferOut: array[0..cryptoBufferSize-1] of byte;
   writeBuffer:array[0..cryptoBufferSize] of byte;
begin
   while (not terminated) and (connected) do
   begin
      try
         logToFile('Encrypting a block of ' + inttostr(sizeof(sendCryptobufferIn)) + ' bytes from microphone');

         // copy bytes from the microphone buffer into sendCryptobuffer
         rawSendBuffer.popBuffer(@sendCryptobufferIn[0], sizeof(sendCryptobufferIn));
         logBytes('unencrypted bytes:', @sendCryptobufferIn, sizeof(sendCryptobufferIn));

         // encrypt the bytes in sendCryptobuffer
         sendCipher.encrypt(sendCryptobufferIn,sendCryptobufferOut,sizeof(sendCryptobufferIn));
         logBytes('encrypted bytes:', @sendCryptobufferOut, sizeof(sendCryptobufferIn));

         // send encrypted bytes in sendCryptobuffer to server
         if connected then // final check
         begin
            logToFile('Writing ' + inttostr(sizeof(sendCryptobufferOut)) + ' audio bytes to stream');

            // add a checksum
            move(sendCryptobufferOut, writeBuffer, sizeof(sendCryptobufferOut));
            writeBuffer[sizeof(writeBuffer)-1] := calcChecksum(@sendCryptobufferIn, sizeof(sendCryptobufferIn));
            outputStream.write(writeBuffer, sizeof(writeBuffer));
            //sleep(50); // can avoid corruption problems
         end
         else
         begin
            logToFile('Not writing audio bytes to stream - disconnected');
         end;
      except
         on E: Exception do
         begin
            logToFile('Error in sender thread: ' + E.message);
         end;
      end;
   end;
end ;


{called by TAudioIn on the main form when it has read a chunk of bytes from the microphone}
function TSenderThread.processBytesFromMicrophone(buffer:pchar; var size: integer): boolean;
var
   compressedSize: cardinal;
   compressedBuffer: array[0..16384] of byte; // in theory this should be smaller than the number of compressed bytes, but sometimes the compressed bytes are larger
   compressedBytesStream: TMemoryStream;
   compressionStream: TCompressionStream;
begin
   logBytes('Bytes from microphone:', pbyte(buffer), size);

   // compress bytes from microphone & place into compressedBuffer
   compressedBytesStream := TMemoryStream.Create;
   compressionStream := TCompressionStream.Create(clMax, compressedBytesStream);
   compressionStream.Write(buffer^, size);
   compressionStream.Free;
   compressedSize := compressedBytesStream.Size;
   assert(compressedSize <= sizeof(compressedBuffer));
   compressedBytesStream.Seek(soFromBeginning,0);
   compressedBytesStream.read(compressedBuffer,compressedSize);
   compressedBytesStream.Free;
   logBytes('Compressed bytes:', @compressedBuffer[0], compressedSize);

   // write compressed size (as a 16-bit number in 2 bytes) into rawSendBuffer
   rawSendBuffer.pushIntegerLittleEndian(compressedSize, 2);

   // write a checksum of the compressed bytes, and the uncompressed bytes
   rawSendBuffer.push(calcChecksum(@compressedBuffer, compressedSize));
   rawSendBuffer.push(calcChecksum(pbyte(buffer), size));

   // write compressed bytes into raw send buffer
   rawSendBuffer.pushBuffer(@compressedBuffer[0], compressedSize);

   logToFile('Dumped ' + inttostr(compressedSize) + ' compressed bytes into raw send buffer, now ' + inttostr(rawSendBuffer.available) + ' bytes available');
   Result := connected;
end;


procedure TSenderThread.clearBuffer;
begin
   rawSendBuffer.clear;
end;

end.
